import 'package:_private_core/_private_core.dart';
import 'package:_private_core_network/network_resources/auth/models/auth_industry.dart';
import 'package:_private_core_network/network_resources/resources.dart';
import 'package:_private_core_network/utils.dart';
import 'package:dio/dio.dart';

import '../model/username_available_result.dart';
import 'models/check_email_result.dart';
import 'models/upload_thumbnail_result.dart';

class _MyAppEndpoint {
  _MyAppEndpoint._();
  static String checkEmail() => "/v2/auth/checkEmail";
  static String verifyEmailOtp() => "/v2/auth/verifyEmailOtp";
  static String loginByEmailVerifiedToken() =>
      "/v2/auth/loginByEmailVerifiedToken";
  static String updateNewEmailByEmailVerifiedToken() =>
      "/v2/auth/updateNewEmailByEmailVerifiedToken";
  static String resetPasswordByEmailVerifiedToken() =>
      "/v2/auth/resetPasswordByEmailVerifiedToken";

  static String setThumbnailForUserOrPro() =>
      "/v2/common/setThumbnailForUserOrPro";

  static String getServiceCatForIndustry() =>
      "v2/common/getServiceCatForIndustry";
  static String loginTerminal() => "/v2/auth/loginTerminal";
  static String loginUserOrProByUsername() =>
      "/v2/auth/loginUserOrProByUsername";
  static String refreshToken() => "/v2/auth/refreshToken";
  static String checkUsernameAvailable() => "v2/auth/checkUsernameAvailable";
  static String createNewPro() => 'v2/auth/createNewPro';
  static String createNewUser() => 'v2/auth/createNewUser';
  static String updateRegistrationLevelForUser() =>
      "/v2/common/updateRegistrationLevelForUser";
  static String updateRegistrationLevelForPro() =>
      "/v2/common/updateRegistrationLevelForPro";

  static String getUserInfo() => "/v2/auth/getUserInfo";
  static String getProInfo() => "/v2/auth/getProInfo";
  static String updateProPhone() => "updateProPhone";
  static String updateUserPhone() => "updateUserPhone";
}

abstract class MyAppApi {
  Future<NetworkResponse> checkEmail(params);
  Future<NetworkResponse> verifyEmailOtp(params);
  Future<NetworkResponse> loginByEmailVerifiedToken(params);
  Future<NetworkResponse> updateNewEmailByEmailVerifiedToken(params);
  Future<NetworkResponse> resetPasswordByEmailVerifiedToken(params);

  Future<NetworkResponse> setThumbnailForUserOrPro(bytes, filePath,
      [ProgressCallback? callback]);

  Future<NetworkResponse> getUserInfo(params);
  Future<NetworkResponse> getProInfo(params);

  Future<NetworkResponse> getServiceCatForIndustry(params);

  Future<NetworkResponse> loginTerminal(params);
  Future<NetworkResponse> loginUserOrProByUsername(params);
  Future<NetworkResponse> refreshToken(refreshToken);

  Future<NetworkResponse> checkUsernameAvailable(params);
  Future<NetworkResponse> createNewPro(params);
  Future<NetworkResponse> createNewUser(params);

  Future<NetworkResponse> updateRegistrationLevelForUser(params);
  Future<NetworkResponse> updateRegistrationLevelForPro(params);
  Future<NetworkResponse> updateProPhone(params);
  Future<NetworkResponse> updateUserPhone(params);
}

class MyAppApiImp extends MyAppApi {
  @override
  Future<NetworkResponse> loginTerminal(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(_MyAppEndpoint.loginTerminal(), data: params);
        return NetworkResponse.fromResponse(response, value: response.data);
      },
    );
  }

  @override
  Future<NetworkResponse> loginUserOrProByUsername(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(_MyAppEndpoint.loginUserOrProByUsername(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => TokenResponseLogin.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> setThumbnailForUserOrPro(bytes, filePath,
      [ProgressCallback? callback]) async {
    return await handleNetworkError(
      proccess: () async {
        String fileName = filePath.split('/').last;
        FormData formData = FormData.fromMap({
          "fileName": MultipartFile.fromBytes(bytes, filename: fileName),
          "ratio": 1,
        });
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(_MyAppEndpoint.setThumbnailForUserOrPro(),
                data: formData, onSendProgress: callback);
        return NetworkResponse.fromResponse(response,
            converter: (json) => UploadThumbnalResult.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> getUserInfo(params) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .get(_MyAppEndpoint.getUserInfo(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => UserModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> getProInfo(params) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .get(_MyAppEndpoint.getProInfo(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => ProInfo.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> getServiceCatForIndustry(params) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .get(_MyAppEndpoint.getServiceCatForIndustry(),
                queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) =>
                (json as List).map((e) => AuthIndustry.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> checkEmail(params) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .post(_MyAppEndpoint.checkEmail(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => CheckEmailResult.fromJson(json));
      },
      builder: (e) {
        if (e.response?.data != null && e.response!.data['error'] is String) {
          return NetworkResponse.fromResponse(e.response!,
              value: CheckEmailResult(message: e.response!.data['error']));
        }
        return NetworkResponse.withErrorRequest(e);
      },
    );
  }

  @override
  Future<NetworkResponse> verifyEmailOtp(params) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .post(_MyAppEndpoint.verifyEmailOtp(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => ConfirmOtp.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> loginByEmailVerifiedToken(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(_MyAppEndpoint.loginByEmailVerifiedToken(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => TokenResponseLogin.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> updateNewEmailByEmailVerifiedToken(params) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .put(_MyAppEndpoint.updateNewEmailByEmailVerifiedToken(),
                data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> resetPasswordByEmailVerifiedToken(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false).put(
            _MyAppEndpoint.resetPasswordByEmailVerifiedToken(),
            data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> refreshToken(refreshToken) async {
    return await handleNetworkError(
      proccess: () async {
        Response response =
            await AppClient(requiredToken: true, token: refreshToken)
                .post(_MyAppEndpoint.refreshToken());
        return NetworkResponse.fromResponse(response,
            converter: (json) => TokenResponseLogin.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> checkUsernameAvailable(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false).get(
            _MyAppEndpoint.checkUsernameAvailable(),
            queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => UsernameAvailableResult.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> createNewPro(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(_MyAppEndpoint.createNewPro(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => TokenResponseLogin.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> createNewUser(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(_MyAppEndpoint.createNewUser(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => TokenResponseLogin.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> updateRegistrationLevelForUser(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(_MyAppEndpoint.updateRegistrationLevelForUser(),
                data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> updateRegistrationLevelForPro(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(_MyAppEndpoint.updateRegistrationLevelForPro(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> updateProPhone(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(_MyAppEndpoint.updateProPhone(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> updateUserPhone(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(_MyAppEndpoint.updateUserPhone(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }
}
